@@ -122,31 +122,39 @@ field. Reflecting this, these subclasses have one additional keyword argument:
 ``location``, which indicates the location to which to redirect.
 """
 
-import types
 from string import Template
 
-from zope.interface import implements
+from zope.interface import implementer
 
 from webob import html_escape as _html_escape
 
+from pyramid.compat import (
+    class_types,
+    text_type,
+    binary_type,
+    text_,
+    )
+
 from pyramid.interfaces import IExceptionResponse
 from pyramid.response import Response
 
 def _no_escape(value):
     if value is None:
         return ''
-    if not isinstance(value, basestring):
+    if not isinstance(value, text_type):
         if hasattr(value, '__unicode__'):
-            value = unicode(value)
+            value = value.__unicode__()
+        if isinstance(value, binary_type):
+            value = text_(value, 'utf-8')
         else:
-            value = str(value)
+            value = text_type(value)
     return value
 
 class HTTPException(Exception): # bw compat
     """ Base class for all :term:`exception response` objects."""
 
+@implementer(IExceptionResponse)
 class WSGIHTTPException(Response, HTTPException):
-    implements(IExceptionResponse)
 
     ## You should set in subclasses:
     # code = 200
@@ -259,7 +267,7 @@ ${body}''')
                     args[k.lower()] = escape(v)
             body = body_tmpl.substitute(args)
             page = page_template.substitute(status=self.status, body=body)
-            if isinstance(page, unicode):
+            if isinstance(page, text_type):
                 page = page.encode(self.charset)
             self.app_iter = [page]
             self.body = page
@@ -370,7 +378,7 @@ class HTTPResetContent(HTTPOk):
     """
     subclass of :class:`~HTTPOk`
 
-    This indicates that the the server has fulfilled the request and
+    This indicates that the server has fulfilled the request and
     the user agent SHOULD reset the document view which caused the
     request to be sent.
     
@@ -429,6 +437,8 @@ ${html_comment}''')
 
     def __init__(self, location='', detail=None, headers=None, comment=None,
                  body_template=None, **kw):
+        if location is None:
+            raise ValueError("HTTP redirects need a location to redirect to.")
         super(_HTTPMove, self).__init__(
             detail=detail, headers=headers, comment=comment,
             body_template=body_template, location=location, **kw)
@@ -744,7 +754,7 @@ class HTTPLengthRequired(HTTPClientError):
     """
     subclass of :class:`~HTTPClientError`
 
-    This indicates that the the server refuses to accept the request
+    This indicates that the server refuses to accept the request
     without a defined Content-Length.
     
     code: 411, title: Length Required
@@ -1016,8 +1026,8 @@ def default_exceptionresponse_view(context, request):
 
 status_map={}
 code = None
-for name, value in globals().items():
-    if (isinstance(value, (type, types.ClassType)) and
+for name, value in list(globals().items()):
+    if (isinstance(value, class_types) and
         issubclass(value, HTTPException)
         and not name.startswith('_')):
         code = getattr(value, 'code', None)
