@@ -1,29 +1,33 @@
+import sys
 import unittest
 
-from pyramid.testing import skip_on
 from pyramid import testing
+from pyramid.compat import text_type
 
 class Base(object):
     def setUp(self):
         self.config = testing.setUp()
-        from zope.deprecation import __show__
-        __show__.off()
 
     def tearDown(self):
         testing.tearDown()
-        from zope.deprecation import __show__
-        __show__.on()
 
     def _getTemplatePath(self, name):
         import os
         here = os.path.abspath(os.path.dirname(__file__))
         return os.path.join(here, 'fixtures', name)
 
-    def _registerUtility(self, utility, iface, name=''):
-        reg = self.config.registry
-        reg.registerUtility(utility, iface, name=name)
-        return reg
-        
+class Test_renderer_factory(Base, unittest.TestCase):
+    def _callFUT(self, info):
+        from pyramid.chameleon_zpt import renderer_factory
+        return renderer_factory(info)
+
+    def test_it(self):
+        # this test is way too functional
+        from pyramid.chameleon_zpt import ZPTTemplateRenderer
+        info = DummyInfo()
+        result = self._callFUT(info)
+        self.assertEqual(result.__class__, ZPTTemplateRenderer)
+
 class ZPTTemplateRendererTests(Base, unittest.TestCase):
     def _getTargetClass(self):
         from pyramid.chameleon_zpt import ZPTTemplateRenderer
@@ -45,17 +49,15 @@ class ZPTTemplateRendererTests(Base, unittest.TestCase):
         from pyramid.interfaces import ITemplateRenderer
         verifyClass(ITemplateRenderer, self._getTargetClass())
 
-    @skip_on('java')
     def test_call(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
         instance = self._makeOne(minimal, lookup)
         result = instance({}, {})
-        self.assertTrue(isinstance(result, unicode))
+        self.assertTrue(isinstance(result, text_type))
         self.assertEqual(result.rstrip('\n'),
                      '<div xmlns="http://www.w3.org/1999/xhtml">\n</div>')
 
-    @skip_on('java')
     def test_template_reified(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
@@ -64,7 +66,6 @@ class ZPTTemplateRendererTests(Base, unittest.TestCase):
         template  = instance.template
         self.assertEqual(template, instance.__dict__['template'])
 
-    @skip_on('java')
     def test_template_with_ichameleon_translate(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
@@ -73,7 +74,6 @@ class ZPTTemplateRendererTests(Base, unittest.TestCase):
         template  = instance.template
         self.assertEqual(template.translate, lookup.translate)
 
-    @skip_on('java')
     def test_template_with_debug_templates(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
@@ -83,7 +83,6 @@ class ZPTTemplateRendererTests(Base, unittest.TestCase):
         template  = instance.template
         self.assertEqual(template.debug, True)
 
-    @skip_on('java')
     def test_template_without_debug_templates(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
@@ -93,7 +92,6 @@ class ZPTTemplateRendererTests(Base, unittest.TestCase):
         template  = instance.template
         self.assertEqual(template.debug, False)
 
-    @skip_on('java')
     def test_template_with_reload_templates(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
@@ -103,7 +101,6 @@ class ZPTTemplateRendererTests(Base, unittest.TestCase):
         template  = instance.template
         self.assertEqual(template.auto_reload, True)
 
-    @skip_on('java')
     def test_template_without_reload_templates(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
@@ -113,102 +110,54 @@ class ZPTTemplateRendererTests(Base, unittest.TestCase):
         template  = instance.template
         self.assertEqual(template.auto_reload, False)
 
-    @skip_on('java')
     def test_call_with_nondict_value(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
         instance = self._makeOne(minimal, lookup)
         self.assertRaises(ValueError, instance, None, {})
 
-    @skip_on('java', 'pypy')
     def test_implementation(self):
         minimal = self._getTemplatePath('minimal.pt')
         lookup = DummyLookup()
         instance = self._makeOne(minimal, lookup)
         result = instance.implementation()()
-        self.assertTrue(isinstance(result, unicode))
+        self.assertTrue(isinstance(result, text_type))
         self.assertEqual(result.rstrip('\n'),
                      '<div xmlns="http://www.w3.org/1999/xhtml">\n</div>')
+
+    def test_macro_supplied(self):
+        minimal = self._getTemplatePath('withmacro.pt')
+        lookup = DummyLookup()
+        instance = self._makeOne(minimal, lookup, macro='foo')
+        result = instance.implementation()()
+        self.assertEqual(result, '\n  Hello!\n')
         
-
-class RenderTemplateTests(Base, unittest.TestCase):
-    def _callFUT(self, name, **kw):
-        from pyramid.chameleon_zpt import render_template
-        return render_template(name, **kw)
-
-    @skip_on('java')
-    def test_it(self):
-        minimal = self._getTemplatePath('minimal.pt')
-        result = self._callFUT(minimal)
-        self.assertTrue(isinstance(result, unicode))
-        self.assertEqual(result.rstrip('\n'),
-                     '<div xmlns="http://www.w3.org/1999/xhtml">\n</div>')
-
-class RenderTemplateToResponseTests(Base, unittest.TestCase):
-    def _callFUT(self, name, **kw):
-        from pyramid.chameleon_zpt import render_template_to_response
-        return render_template_to_response(name, **kw)
-
-    @skip_on('java')
-    def test_it(self):
-        minimal = self._getTemplatePath('minimal.pt')
-        result = self._callFUT(minimal)
-        from webob import Response
-        self.assertTrue(isinstance(result, Response))
-        self.assertEqual(result.app_iter[0].rstrip('\n'),
-                     '<div xmlns="http://www.w3.org/1999/xhtml">\n</div>')
-        self.assertEqual(result.status, '200 OK')
-        self.assertEqual(len(result.headerlist), 2)
-
-    @skip_on('java')
-    def test_iresponsefactory_override(self):
-        from webob import Response
-        class Response2(Response):
-            pass
-        from pyramid.interfaces import IResponseFactory
-        self._registerUtility(Response2, IResponseFactory)
-        minimal = self._getTemplatePath('minimal.pt')
-        result = self._callFUT(minimal)
-        self.assertTrue(isinstance(result, Response2))
-
-class GetRendererTests(Base, unittest.TestCase):
-    def _callFUT(self, name):
-        from pyramid.chameleon_zpt import get_renderer
-        return get_renderer(name)
-
-    @skip_on('java')
-    def test_it(self):
-        from pyramid.interfaces import IRendererFactory
-        class Dummy:
-            template = object()
-            def implementation(self): pass
-        renderer = Dummy()
-        def rf(spec):
-            return renderer
-        self._registerUtility(rf, IRendererFactory, name='foo')
-        result = self._callFUT('foo')
-        self.assertTrue(result is renderer)
-
-class GetTemplateTests(Base, unittest.TestCase):
-    def _callFUT(self, name):
-        from pyramid.chameleon_zpt import get_template
-        return get_template(name)
-
-    @skip_on('java')
-    def test_it(self):
-        from pyramid.interfaces import IRendererFactory
-        class Dummy:
-            template = object()
-            def implementation(self):
-                return self.template
-        renderer = Dummy()
-        def rf(spec):
-            return renderer
-        self._registerUtility(rf, IRendererFactory, name='foo')
-        result = self._callFUT('foo')
-        self.assertTrue(result is renderer.template)
+    def test_macro_notsupplied(self):
+        minimal = self._getTemplatePath('withmacro.pt')
+        lookup = DummyLookup()
+        instance = self._makeOne(minimal, lookup)
+        result = instance.implementation()()
+        self.assertEqual(result,
+                         '<html>\nOutside macro\n\n  Hello!\n\n</html>\n\n')
 
 class DummyLookup(object):
     auto_reload=True
     debug = True
     def translate(self, msg): pass
+
+class DummyRegistry(object):
+    def queryUtility(self, iface, name):
+        self.queried = iface, name
+        return None
+
+    def registerUtility(self, impl, iface, name):
+        self.registered = impl, iface, name
+    
+class DummyInfo(object):
+    def __init__(self):
+        self.registry = DummyRegistry()
+        self.type = '.pt'
+        self.name = 'fixtures/minimal.pt'
+        self.package = sys.modules[__name__]
+        self.settings = {}
+    
